# Generated by Django 4.2.3 on 2023-07-19 10:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('webkassa', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='check',
            name='order_number',
            field=models.CharField(default=1, max_length=255, unique=True, verbose_name='Order number'),
            preserve_default=False,
        ),
    ]
