# Generated by Django 4.2.3 on 2023-07-20 06:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('webkassa', '0003_remove_webkassaaccount_is_encrypted_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webkassaaccount',
            name='email',
            field=models.EmailField(max_length=254),
        ),
        migrations.AlterUniqueTogether(
            name='webkassaaccount',
            unique_together={('email', 'cashbox_unique_number')},
        ),
    ]
