# Generated by Django 4.2.3 on 2023-07-20 04:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('webkassa', '0002_check_order_number'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='webkassaaccount',
            name='is_encrypted',
        ),
    ]
